<?php

namespace SGalinski\SgMail\Service;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Cache\CacheManager;
use TYPO3\CMS\Core\Cache\Frontend\FrontendInterface;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class RegisterService
 *
 * @package SGalinski\SgMail\Service
 */
class RegisterService implements \TYPO3\CMS\Core\SingletonInterface {
	const CACHE_NAME = 'sg_mail_registerArrayCache';
	const CACHE_LIFETIME_IN_SECONDS = 86400;

	/**
	 * @var array
	 */
	public $registerArray = [];

	/**
	 * @var array
	 */
	private $registrationFiles = [];

	/**
	 * Enables extensions to add their registration files
	 *
	 * @param string $pathToRegisterFile
	 */
	public function register($pathToRegisterFile) {
		if (file_exists($pathToRegisterFile)) {
			$this->registerArray[] = [
				'path' => $pathToRegisterFile
			];
		}
	}

	/**
	 * Get all registered templates from the cache
	 *
	 * @return array
	 * @throws \TYPO3\CMS\Core\Cache\Exception\NoSuchCacheException
	 * @throws \BadFunctionCallException
	 * @throws \InvalidArgumentException
	 */
	public function getRegisterArray(): array {
		/** @var CacheManager $cacheManager */
		$cacheManager = GeneralUtility::makeInstance(CacheManager::class);
		/** @var FrontendInterface $cache */
		$cache = $cacheManager->getCache(self::CACHE_NAME);
		$cacheId = md5('sg_mail');
		/** @var array entry */
		if (($entry = $cache->get($cacheId)) === FALSE) {
			$entry = $this->registerExtensions();

			if ($entry === NULL) {
				$entry = [];
			}

			$cache->set($cacheId, $entry, [], self::CACHE_LIFETIME_IN_SECONDS);
		}

		return $entry;
	}

	/**
	 * Read every registered file and create a registration entry in the registerArray if possible
	 */
	private function registerExtensions() {
		foreach ($this->registrationFiles as $registerFile) {

			// clear registerArray
			$registerArray = [];

			if (\is_file($registerFile)) {
				$configArray = (include $registerFile);

				$extensionKey = $configArray['extension_key'];
				$templateKey = $configArray['template_key'];

				if ($extensionKey === NULL || $templateKey === NULL) {
					continue;
				}

				$registerArray = $this->writeRegisterArrayEntry(
					$registerArray, $extensionKey, $templateKey, $configArray
				);
			}

			// set the register array after every registration has been parsed
			$this->registerArray = $registerArray;
		}
	}

	/**
	 * write a single entry into the register array
	 *
	 * @param array $registerArray
	 * @param $extensionKey
	 * @param $templateKey
	 * @param array $configArray
	 * @param bool $transformTemplateFolder
	 * @param string $storeTemplateExtension
	 * @return array
	 */
	private function writeRegisterArrayEntry(
		array $registerArray, $extensionKey, $templateKey, array $configArray,
		$transformTemplateFolder = TRUE, $storeTemplateExtension = ''
	): array {
		// If it is not explicitly set in which extension the html should be located, use the extension set in the template settings
		if ($storeTemplateExtension === '') {
			$storeTemplateExtension = $extensionKey;
		}

		// give the option to use the template key as folder name. this is used mainly with auto registering
		$templateDirectory = $templateKey;

		// by default folders with underscore will be transformed to upper camelcase
		if ($transformTemplateFolder) {
			// transform template directory name: your_templates => YourTemplates/
			$templateDirectoryParts = GeneralUtility::trimExplode('_', $templateKey);
			$templateDirectory = '';
			foreach ($templateDirectoryParts as $part) {
				$templateDirectory .= ucfirst($part);
			}
		}

		$templateDirectory .= '/';
		$templatePath = ExtensionManagementUtility::extPath($storeTemplateExtension) . self::DEFAULT_TEMPLATE_PATH
			. $templateDirectory;

		if ($configArray['template_path']) {
			$templatePath = $configArray[$templateKey];
		}

		$description = $configArray['description'];
		$subject = $configArray['subject'];
		$marker = $configArray['markers'];

		$registerArray[$extensionKey][$templateKey] = [
			'templatePath' => $templatePath,
			'description' => $description,
			'marker' => $marker,
			'extension' => $extensionKey,
			'templateName' => $templateKey,
			'subject' => $subject
		];

		return $registerArray;
	}
}

